/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description:Contains the definition of the CDatabase class. 
*/

#ifndef CDATABASE_H_
#define CDATABASE_H_

#include <e32cons.h>
#include <e32base.h>  //Used for CBase.
#include <d32dbms.h>  //Used for RDbDatabase class.
#include <s32file.h>  //Used for CFileStore class.

/**
This is a database class which is used by two different threads to demonstrate thread
synchronization using a semaphore. 
*/
class CDatabase : public CBase
{
public:
    static CDatabase* NewL();
    virtual ~CDatabase();
    
    //Method to create a database.
    void CreateDatabaseL();
    //Method to read the database. 
    void ReadfromDatabaseL();
    //Method to write into the database.
    void WritetoDatabaseL();
    
    //The callback function for ReaderThread associated with the CPeriodic object.
    static TInt PeriodicReadL(TAny* aPtr);
    //The periodic function to read the database. 
    void PeriodicReadFuncL();
    
    //The callback function for WriterThread associated with the CPeriodic object.
    static TInt PeriodicWriteL(TAny* aPtr);
    //The periodic function to write into the database.
    void PeriodicWriteFuncL();

protected: 
    void ConstructL();

private: 
    RFs iFsSession; //file session object.
    RDbStoreDatabase iDatabase; //database object.
    TFileName iDbname;
    
    //CPeriodic object to periodically read/write to the DB.
    CPeriodic* iPeriodic; 
    
    CConsoleBase* iConsole; 
    
    #ifdef USE_SEMAPHORE
    //Semaphore which schedules order of execution.  
    RSemaphore iSemWrite; 
    #endif
};

#endif /*CDATABASE_H_*/
